@@ -79,7 +79,8 @@ class HostOpenStack(Host):
     def __init__(self, config_dir, share_dir, **kwargs):
         super().__init__(config_dir, share_dir, **kwargs)
         self.args = kwargs
-        self.dotenough = dotenough.DotEnoughOpenStack(config_dir, self.args['domain'])
+        self.dotenough = dotenough.DotEnoughOpenStack(config_dir, self.args['domain'],
+                                                      self.args['provider'])
         self.dotenough.ensure()
 
     def create_or_update(self):
