@@ -66,6 +66,13 @@ class Service(object):
             if internal_hosts & hosts:
                 self.service2hosts[service] = list(internal_hosts | openvpn_hosts)
 
+    def get_vpn_host(self):
+        hosts = self.service2group.get('openvpn')
+        if hosts:
+            return hosts[0]
+        else:
+            return None
+
     def add_vpn_hosts_if_needed(self, hosts):
         internal_hosts = set(self.hosts_with_internal_network(hosts))
         if not internal_hosts:
@@ -76,8 +83,8 @@ class Service(object):
         return list(hosts | openvpn_hosts)
 
     def hosts_with_internal_network(self, hosts):
-        info = self.ansible.get_variable_hosts('encrypted_device', 'network_internal_only', *hosts)
-        return [host for (host, internal_only) in info.items() if internal_only == 'True']
+        info = self.ansible.get_hostvars('network_internal_only', *hosts)
+        return [host for (host, internal_only) in info.items() if internal_only is True]
 
     def service_from_host(self, host):
         found = None
@@ -105,7 +112,8 @@ class ServiceOpenStack(Service):
     def __init__(self, config_dir, share_dir, **kwargs):
         super().__init__(config_dir, share_dir, **kwargs)
         self.args = kwargs
-        self.dotenough = dotenough.DotEnoughOpenStack(config_dir, self.args['domain'])
+        self.dotenough = dotenough.DotEnoughOpenStack(config_dir, self.args['domain'],
+                                                      self.args['provider'])
         self.dotenough.ensure()
 
     def maybe_delegate_dns(self):
