@@ -8,8 +8,7 @@ IcingaHelper.icinga_host = 'bind-host'
 class TestChecks(IcingaHelper):
 
     def test_host(self):
-        r = self.get_client().objects.get('Host', 'jitsi-host')
-        assert r['attrs']['name'] == 'jitsi-host'
+        assert 'jisti-host' in self.get_hosts(host='jisti-host')
 
     def test_service(self, host):
         assert self.is_service_ok('jitsi-host!Jitsi')
