@@ -5,7 +5,7 @@ import yaml
 
 def get_address(inventory):
     vars_dir = f'{inventory}/group_vars/all'
-    return 'https://wordpress.' + yaml.load(
+    return 'https://wordpress.' + yaml.safe_load(
         open(vars_dir + '/domain.yml'))['domain']
 
 
