@@ -137,7 +137,7 @@ if [ -z "$OS_REGION_NAME" ]; then unset OS_REGION_NAME; fi
 
     assert DotEnoughOpenStack.openrc2clouds(
         openrc_file, clouds_file) == expected['clouds']['production']
-    assert yaml.load(open(clouds_file)) == expected
+    assert yaml.safe_load(open(clouds_file)) == expected
     os.unlink(openrc_file)
     os.unlink(clouds_file)
 
@@ -185,7 +185,7 @@ export OS_IDENTITY_API_VERSION=3
 
     assert DotEnoughOpenStack.openrc2clouds(
         openrc_file, clouds_file) == expected['clouds']['production']
-    assert yaml.load(open(clouds_file)) == expected
+    assert yaml.safe_load(open(clouds_file)) == expected
     os.unlink(clouds_file)
 
     expected['clouds']['production']['auth']['password'] = 'PLACEHOLDER'
@@ -202,7 +202,7 @@ export OS_REGION_NAME="{region_name}"
 
     assert DotEnoughOpenStack.openrc2clouds(
         openrc_file, clouds_file) == expected['clouds']['production']
-    assert yaml.load(open(clouds_file)) == expected
+    assert yaml.safe_load(open(clouds_file)) == expected
     os.unlink(clouds_file)
 
     assert DotEnoughOpenStack.set_missing_config_file_from_openrc(openrc_file, clouds_file) is True
