@@ -1,5 +1,6 @@
 import datetime
 import os
+
 import pytest
 import requests_mock as requests_mock_module
 
@@ -13,22 +14,20 @@ from enough.common.openstack import Stack, Heat, OpenStack
 #
 # Stack
 #
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_stack_create_or_update(openstack_name):
+@pytest.mark.openstack_integration
+def test_stack_create_or_update(openstack_name, openstack_provider,
+                                dot_openstack):
     name = openstack_name
     network = openstack_name
     class_c = '10.101.30'
     cidr = f'{class_c}.0/24'
-    d = {
-        'name': name,
-        'flavor': 's1-2',
-        'port': '22',
-        'internal_network': network,
-        'internal_network_cidr': cidr,
-    }
+    d = Heat.hostvars_to_stack(openstack_name, openstack_provider)
+    d.update(
+        internal_network=network,
+        internal_network_cidr=cidr
+    )
     s = Stack(settings.CONFIG_DIR, d)
-    s.set_public_key('infrastructure_key.pub')
+    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
     r = s.create_or_update()
     assert r['port'] == '22'
     ipv4 = r['ipv4']
@@ -96,9 +95,23 @@ def test_create_test_subdomain_with_bind(tmpdir, mocker, requests_mock):
 #
 # OpenStack
 #
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_network(openstack_name):
+def test_server_ip_in_network_ok(mocker):
+    ip = '1.2.3.4'
+    mocker.patch('enough.common.openstack.OpenStack.server_port_list',
+                 return_value="[{'ip_address': '" + ip + "'}, {'ip_address': '1:2:3:4:5'}]")
+    o = OpenStack(settings.CONFIG_DIR)
+    assert o.server_ip_in_network('server', 'network') == ip
+
+
+def test_server_ip_in_network_none(mocker):
+    mocker.patch('enough.common.openstack.OpenStack.server_port_list',
+                 return_value='[]')
+    o = OpenStack(settings.CONFIG_DIR)
+    assert o.server_ip_in_network('server', 'network') is None
+
+
+@pytest.mark.openstack_integration
+def test_network(openstack_name, dot_openstack):
     o = OpenStack(settings.CONFIG_DIR)
     o.network_and_subnet_create(openstack_name, '10.11.12.0/24')
     assert o.network_exists(openstack_name)
@@ -107,12 +120,11 @@ def test_network(openstack_name):
     dns_ip = '1.2.3.4'
     o.subnet_update_dns(openstack_name, dns_ip)
     r = o.o.subnet.show('--format=value', '-c', 'dns_nameservers', openstack_name)
-    assert r.strip() == dns_ip
+    assert r.strip() == f"['{dns_ip}']"
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_backup_create_with_name(openstack_name, caplog):
+@pytest.mark.openstack_integration
+def test_backup_create_with_name(openstack_name, dot_openstack, caplog):
     o = OpenStack(settings.CONFIG_DIR)
     o.o.volume.create('--size=1', openstack_name)
     assert o.backup_create([openstack_name]) == 1
@@ -123,9 +135,8 @@ def test_backup_create_with_name(openstack_name, caplog):
     assert o.backup_prune(0) == 0
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_volume_prune(openstack_name, caplog):
+@pytest.mark.openstack_integration
+def test_volume_prune(openstack_name, dot_openstack, caplog):
     o = OpenStack(settings.CONFIG_DIR)
 
     volume_precious = openstack_name
@@ -155,9 +166,8 @@ def test_volume_prune(openstack_name, caplog):
                                     'recent': [recent_volume]}
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_backup_create_no_names(openstack_name, caplog):
+@pytest.mark.openstack_integration
+def test_backup_create_no_names(openstack_name, dot_openstack, caplog):
     o = OpenStack(settings.CONFIG_DIR)
     o.o.volume.create('--size=1', openstack_name)
     o.backup_create([])
@@ -165,22 +175,20 @@ def test_backup_create_no_names(openstack_name, caplog):
     assert available_snapshot in caplog.text
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_openstack_replace_volume(openstack_name):
-    d = {
-        'name': openstack_name,
-        'flavor': 's1-2',
-        'port': '22',
+@pytest.mark.openstack_integration
+def test_openstack_replace_volume(openstack_name, dot_openstack,
+                                  openstack_provider):
+    d = Heat.hostvars_to_stack(openstack_name, openstack_provider)
+    d.update({
         'volumes': [
             {
                 'size': '1',
                 'name': openstack_name,
             },
         ],
-    }
+    })
     s = Stack(settings.CONFIG_DIR, d)
-    s.set_public_key('infrastructure_key.pub')
+    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
     s.create_or_update()
 
     o = OpenStack(settings.CONFIG_DIR)
@@ -207,23 +215,21 @@ def test_openstack_replace_volume(openstack_name):
         '-f=value', '-c=Name', '--name', openstack_name).strip() == openstack_name
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_openstack_volume_resize_ok(openstack_name):
+@pytest.mark.openstack_integration
+def test_openstack_volume_resize_ok(openstack_name, dot_openstack,
+                                    openstack_provider):
     size = 1
-    d = {
-        'name': openstack_name,
-        'flavor': 's1-2',
-        'port': '22',
+    d = Heat.hostvars_to_stack(openstack_name, openstack_provider)
+    d.update({
         'volumes': [
             {
                 'size': str(size),
                 'name': openstack_name,
             },
         ],
-    }
+    })
     s = Stack(settings.CONFIG_DIR, d)
-    s.set_public_key('infrastructure_key.pub')
+    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
     s.create_or_update()
 
     o = OpenStack(settings.CONFIG_DIR)
@@ -246,16 +252,11 @@ def test_openstack_volume_resize_ok(openstack_name):
         o.volume_resize(openstack_name, openstack_name, 1)
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_openstack_volume_resize_no_volume(openstack_name):
-    d = {
-        'name': openstack_name,
-        'flavor': 's1-2',
-        'port': '22',
-    }
+@pytest.mark.openstack_integration
+def test_openstack_volume_resize_no_volume(openstack_name, dot_openstack, openstack_provider):
+    d = Heat.hostvars_to_stack(openstack_name, openstack_provider)
     s = Stack(settings.CONFIG_DIR, d)
-    s.set_public_key('infrastructure_key.pub')
+    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
     s.create_or_update()
 
     o = OpenStack(settings.CONFIG_DIR)
@@ -264,10 +265,20 @@ def test_openstack_volume_resize_no_volume(openstack_name):
         o.volume_resize(openstack_name, openstack_name, 1)
 
 
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_openstack_security_group(openstack_name):
+@pytest.mark.openstack_integration
+def test_openstack_security_group(openstack_name, dot_openstack):
     o = OpenStack(settings.CONFIG_DIR)
     o.o.security.group.create(openstack_name)
     assert o.delete_security_group(openstack_name) is True
     assert o.delete_security_group(openstack_name) is False
+
+
+@pytest.mark.openstack_integration
+def test_destroy_volumes_with_same_name(openstack_name, dot_openstack):
+    volume_name = f'{openstack_name}-test-destroy-volume'
+    o = OpenStack(settings.CONFIG_DIR)
+    o.o.volume.create('--size', '1', volume_name)
+    o.o.volume.create('--size', '1', volume_name)
+    assert 2 == o.o.volume.list().count(volume_name)
+    o.destroy_everything(openstack_name)
+    assert 0 == o.o.volume.list().count(volume_name)
