@@ -19,7 +19,9 @@ function prepare_environment() {
 
 function prepare_repository() {
     git submodule update --init --recursive
-    chmod 600 infrastructure_key
+    if test -f infrastructure_key ; then
+        chmod 600 infrastructure_key
+    fi
 }
 
 function prepare_inventory() {
@@ -118,12 +120,10 @@ function run_tests() {
     prepare_repository
 
     find $d \( -name '*.pyc' -o -name '*.pyo' -o -name __pycache__ \) -delete
-    skip=""
-    if test "$SKIP_OPENSTACK_INTEGRATION_TESTS" ; then
-        skip+="-e SKIP_OPENSTACK_INTEGRATION_TESTS=true "
-    fi
     if test "$GITLAB_CI" ; then
         args="--workdir /opt"
+        : ${PYTEST_ADDOPTS=-m \"not openstack_integration\"}
+        PASS_GITLAB_CI="-e GITLAB_CI=${GITLAB_CI}"
     else
         args="--volume ${d}:${d} --workdir ${d} --volume $HOME/.enough:$HOME/.enough --volume $HOME/.ansible:$HOME/.ansible -e HOME=$HOME"
         # handle git worktree
@@ -132,8 +132,10 @@ function run_tests() {
             git_common_dir="$(git rev-parse --git-common-dir)"
             args="$args --volume ${git_common_dir}:${git_common_dir}"
         fi
+        PASS_GITLAB_CI=""
+	local interactive=-ti
     fi
-    docker run --rm --device=/dev/net/tun --name $name --user "${USER:-root}" -e ENOUGH_API_TOKEN=$ENOUGH_API_TOKEN $skip --cap-add=NET_ADMIN $args --volume /var/run/docker.sock:/var/run/docker.sock $name "${@:-tox}"
+    docker run $interactive --rm --device=/dev/net/tun --name $name --user "${USER:-root}" -e PYTEST_ADDOPTS="${PYTEST_ADDOPTS-}" -e ENOUGH_API_TOKEN=$ENOUGH_API_TOKEN ${PASS_GITLAB_CI} --cap-add=NET_ADMIN $args --volume /var/run/docker.sock:/var/run/docker.sock $name "${@:-tox}"
 }
 
 function main() {
